#!/bin/bash
#
# Description:
#   A script for replaying traces (uses APTR from the DA-BMC tool chain)
# Author:
#   Jan Fiedor
# Version:
#   1.1
# Created:
#   20.06.2011
# Last Update:
#   21.06.2011
#

# Helper functions section
# ------------------------

#
# Description:
#   Prints an error message with preceding red 'error: ' to a console and
#   normal message with preceding 'error: ' otherwise
# Parameters:
#   [STRING] A message
# Return:
#   Nothing
#
errmess()
{
  if [ -c /dev/stderr ]; then
    echo -e "\e[1;31merror: \e[0m$1" 1>&2
  else
    echo "error: "$1 1>&2
  fi
}

# Configuration section
# ---------------------

# Load settings
source ./run-aptr.cfg

# Check if path to RunJPF.jar specified
if [ -z "$RUNJPF" ]; then
  # Path to RunJPF.jar not specified, try to resolve it from the site.properties
  JPF_HOME=`cat ~/.jpf/site.properties | grep -E '^[\t ]*jpf[.]home' | sed -E 's/^[\t ]*jpf[.]home[\t ]*=[\t ]*//g' | sed -E "s,[$][{]user[.]home[}],$HOME,g"`

  # Check if the extracted JPF's home directory is a valid directory
  if [ ! -d $JPF_HOME ]; then
    errmess "cannot resolve a path to jpf's home directory, '"$JPF_HOME"' is not a valid directory!"
    exit 1
  fi

  # Get a path to a home directory of the core part of JPF (jpf-core)
  JPF_CORE=`cat ~/.jpf/site.properties | grep -E '^[\t ]*jpf-core' | sed -E 's/^[\t ]*jpf-core[\t ]*=[\t ]*//g' | sed -E "s,[$][{]jpf[.]home[}],$JPF_HOME,g"`

  # Check if the extracted JPF Core's home directory is a valid directory
  if [ ! -d $JPF_CORE ]; then
    errmess "cannot resolve a path to jpf-core's home directory, '"$JPF_CORE"' is not a valid directory!"
    exit 1
  fi

  # Get a path to the RunJPF.jar file
  RUNJPF=$JPF_CORE/build/RunJPF.jar
fi

# Check if RunJPF.jar present
if [ ! -f $RUNJPF ]; then
  errmess "$RUNJPF not found!"
  exit 1
fi

# Check if dev-libs.jar present
if [ ! -f $DA_BMC_LIBS/dev-libs.jar ]; then
  errmess "dev-libs.jar not found!"
  exit 1
fi

# Check if aptr.jar present
if [ ! -f $DA_BMC_LIBS/aptr.jar ]; then
  errmess "aptr.jar not found!"
  exit 1
fi

# Check if file containing the ConTest Method to Block Type table is present
if [ ! -f $DA_BMC_LIBS/tables/ctm2bt.table ]; then
  errmess "$DA_BMC_LIBS/tables/ctm2bt.table not found!"
  exit 1
fi

# Check if file containing the ConTest Method to Location Type table is present
if [ ! -f $DA_BMC_LIBS/tables/ctm2lt.table ]; then
  errmess "$DA_BMC_LIBS/tables/ctm2lt.table not found!"
  exit 1
fi

# Check if trace file specified
if [ -z "$1" ]; then
  errmess "no trace file specified!"
  exit 1
else
  TRACE_FILE=$1
fi

shift

# Check if application main class specified
if [ -z "$1" ]; then
  errmess "no application main class specified!"
  exit 1
else
  APP_MAIN_CLASS=$1
fi

shift

# Check if directories containing application class files specified
if [ "$1" != "-cp" ]; then
  APP_CLASSPATH=.
else
  shift

  APP_CLASSPATH=$1

  shift
fi

# Check if directory containing application class files exist
if [ ! -d $APP_CLASSPATH ]; then
  errmess "invalid directory "$APP_CLASSPATH"!"
  exit 1
fi

# Program section
# ---------------

# Replay the specified trace
java $JVM_FLAGS -jar $RUNJPF +shell.port=4242 +target=$APP_MAIN_CLASS +native_classpath+="$DA_BMC_LIBS/dev-libs.jar;$DA_BMC_LIBS/aptr.jar;$APP_CLASSPATH" +verifit.aptr.contest.analyser.ConTestInstructionSkipAnalyser.table.ctMethodToBlockType.file="$DA_BMC_LIBS/tables/ctm2bt.table" +verifit.aptr.contest.table.ctMethodToLocationType.file="$DA_BMC_LIBS/tables/ctm2lt.table" +verifit.aptr.listener.SSnavigator.trace.file="$TRACE_FILE" ./run-aptr.jpf "$@"
